from datetime import date

from django import forms
from django.forms.widgets import Textarea, SelectDateWidget

from captcha.fields import CaptchaField
from django.utils.safestring import mark_safe

from gestion.fields import AddressWidget
from gestion.inscriptions.models import MODELES_CHAMP, DonneeFormulaire, get_filieres
from .models import InscriptionLimitee

annee = date.today().year
choix_promo = [(str(i), f"{i} ({annee + 4 -i}A)" if i <= annee + 3 else str(i)) for i in range(annee, annee + 6)]


class InscriptionLimiteeForm(forms.ModelForm):
    def __init__(self, ecole, *args, **kwargs):
        super(InscriptionLimiteeForm, self).__init__(*args, **kwargs)

        instance = kwargs.get("instance", None)

        for type_champ in MODELES_CHAMP.values():
            for champ in type_champ.objects.filter(ecole=ecole, inscription_directe=True):
                valeur = None
                if instance is not None:
                    try:
                        valeur = DonneeFormulaire.objects.get(inscription=instance, ident_champ=champ.ident).valeur
                    except DonneeFormulaire.DoesNotExist:
                        pass

                champ.ajouter_a(self, valeur)

        self.fields["captcha"] = CaptchaField()

        filieres = get_filieres(ecole)
        filieres = (("", "----------"),) + tuple(filieres)

        self.fields["filiere"] = forms.ChoiceField(required=True, label="Filière", choices=filieres)

        self.fields["promo"] = forms.ChoiceField(
            required=True, label="Promo", initial=str(annee + 3), choices=choix_promo
        )

        self.fields["charte"] = forms.BooleanField(
            required=True,
            label=mark_safe(
                "Je signe <a target='_blank' rel='noopener noreferrer' href='https://inp-net.eu.org/docs/charte_inp-net_2018.pdf'>la charte INP-net</a>."
            ),
            initial=False,
        )

    def clean_nom(self):
        nom = self.cleaned_data["nom"].strip()

        if nom.lower() == nom or (len(nom) > 1 and nom.upper() == nom):
            raise forms.ValidationError(
                "Capitalisation incorrecte du nom : "
                "Il ne doit pas être entièrement en majuscules ou en "
                "minuscules."
            )

        return nom

    def clean_prenom(self):
        prenom = self.cleaned_data["prenom"].strip()

        if prenom.lower() == prenom or (len(prenom) > 1 and prenom.upper() == prenom):
            raise forms.ValidationError(
                "Capitalisation incorrecte du "
                "prénom : Il ne doit pas être entièrement en majuscules ou en "
                "minuscules."
            )

        return prenom

    class Meta:
        model = InscriptionLimitee
        exclude = ()
        widgets = {
            "adresse": AddressWidget,
            "adr_parents": Textarea(attrs={"rows": 4}),
            "date_naissance": SelectDateWidget(years=range(annee - 45, annee - 10)),
        }
